% simulated annealing
% maximization problem

function [samples, Hbar, xbar] = SA_dist4(N, K, choices, M)
% N: # of samples
% K: # of iterations
% choices: choice of test function
% M: dimension of the problem

% intialization
% X is a N*M matrix, each row is a sample
D=41;  %number of discrete points at 1 dim
dist=0.5;  %distance between points
st=-10;  %start point
X_k_indx=unidrnd(D,N,M);
X_k=st+(X_k_indx-1)*dist;

T = 5*10^2;    % initial temperature
Hbar(1) = -inf;
samples(1)=0;

for i = 2:1:K
    samples(i) = (i-1)*N;
    % change temperature
    T1=5*10^(2)/log(i+1);
    %T1=1e-5;
    %T1=10^(-15)+10^(-1)*abs(Hbar(i-1))/log(i+1);
    %T1 = 1/(1+1*10^(-2)*T)*T+1e-10;
    % evaluate weights
     switch choices
        case 1
            Hl=0;
            G=-rosenbrock(X_k)+Hl;
        case 2
            Hl=0;
            G =-tri(X_k)+Hl;
        case 3
            Hl=0;
            G=-griewank(X_k)+Hl;
        case 4
            Hl=0;
            G=shekel(X_k)+Hl;
        case 5
            Hl=0;
            G=-dejong5(X_k)+Hl;
        case 6
            Hl=0;
            G=-powel(X_k)+Hl;
        case 7
            Hl=0;
            G=-levy(X_k)+Hl;
        case 8 
            Hl=0;
            G=-rastrigin(X_k)+Hl;
        case 9
            Hl=0;
            G = -pinter(X_k)+Hl;
        case 10
            Hl=0;
            G = -sphere(X_k)+Hl;           
    end  % end switch
    
    % estimate max H and x in this interation
    [Hbar(i), indx] = max(G);
    Hbar(i)=Hbar(i)-Hl;
    xbar(i,:) = X_k(indx,:);
    
    if i>2
    if Hbar(i-1)>max(G)     
        Hbar(i)=Hbar(i-1);
        xbar(i,:) =xbar(i-1,:);
    end 
    end
 
   
    
    % MCMC move
    % generate Y
    ind=unidrnd(M);
    Y_d1=zeros(N,M);
    Y_d1(:,ind)=(unidrnd(2,N,1)*2-3)*dist;
    %Y_d1=(unidrnd(2,N,M)*2-3)*dist;
    
    Y_d2=st+(unidrnd(D,N,M)-1)*dist;
    U1 = rand(N,1);
    Y_d = Y_d1.*((U1 <= 0.99)*ones(1,M)) + Y_d2.*((U1>0.99)*ones(1,M));
    Y = X_k + Y_d;
    %Y = X + randn(N,M);
    % acceptance prob
    switch choices
        case 1
            dG=-rosenbrock(Y)+rosenbrock(X_k);
        case 2
            dG = -tri(Y)+tri(X_k);
        case 3
            dG= -griewank(Y)+griewank(X_k);
        case 4
            dG=shekel(Y)-shekel(X_k);
        case 5
            dG=-dejong5(Y)+dejong5(X_k);
        case 6
            dG=-powel(Y)+powel(X_k);
        case 7
            dG=-levy(Y)+levy(X_k);
        case 8 
            dG=-rastrigin(Y)+rastrigin(X_k);
        case 9
            dG = -pinter(Y)+pinter(X_k);
        case 10
            dG = -sphere(Y)+sphere(X_k);
    end
    rho = min(exp(dG/T1),1);
    % accept & reject
    U = rand(N,1);
    X_k = Y.*((U <= rho)*ones(1,M)) + X_k.*((U>rho)*ones(1,M));
    T=T1;
    %fprintf('%d   %12.10f \n',i, Hbar(i));
end
 % T  
%figure;
%semilogy([1:1:K], Hbar);